"""Annotate check

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from typing import List, Any

from rougail.annotator.target import TargetAnnotator
from rougail.annotator.param import ParamAnnotator

from rougail.utils import load_modules
from rougail.i18n import _
from rougail.error import DictConsistencyError, display_xmlfiles

INTERNAL_FUNCTIONS = ['valid_in_network', 'valid_differ', 'valid_entier']

class Annotator(TargetAnnotator, ParamAnnotator):
    """Annotate check
    """
    level = 40
    def __init__(self,
                 objectspace,
                 eosfunc_file,
                 *args,
                 ):
        if not hasattr(objectspace.space, 'constraints') or \
                not hasattr(objectspace.space.constraints, 'check'):
            return
        self.objectspace = objectspace
        self.let_none = True
        self.only_variable = True
        self.functions = dir(load_modules(eosfunc_file))
        self.functions.extend(INTERNAL_FUNCTIONS)
        self.functions.extend(self.objectspace.rougailconfig['internal_functions'])
        self.target_is_uniq = False
        self.allow_function = True
        self.convert_target(self.objectspace.space.constraints.check)
        self.convert_param(self.objectspace.space.constraints.check)
        self.check_check()
        self.check_change_warning()
        self.convert_valid_entier()
        self.convert_check()
        del objectspace.space.constraints.check

    def check_check(self):  # pylint: disable=R0912
        """valid and manage <check>
        """
        remove_indexes = []
        for check_idx, check in enumerate(self.objectspace.space.constraints.check):
            if not check.name in self.functions:
                msg = _(f'cannot find check function "{check.name}"')
                raise DictConsistencyError(msg, 1, check.xmlfiles)
            if hasattr(check, 'param') and check.param == []:
                remove_indexes.append(check_idx)
        remove_indexes.sort(reverse=True)
        for idx in remove_indexes:
            del self.objectspace.space.constraints.check[idx]

    def check_change_warning(self):
        """convert level to "warnings_only"
        """
        for check in self.objectspace.space.constraints.check:
            check.warnings_only = check.level == 'warning'
            check.level = None

    def convert_valid_entier(self) -> None:
        """valid and manage <check>
        """
        remove_indexes = []
        for check_idx, check in enumerate(self.objectspace.space.constraints.check):
            if not check.name == 'valid_entier':
                continue
            remove_indexes.append(check_idx)
            if not hasattr(check, 'param'):
                msg = _(f'{check.name} must have, at least, 1 param')
                raise DictConsistencyError(msg, 17, check.xmlfiles)
            for param in check.param:
                if param.type != 'number':
                    msg = _(f'param in "valid_entier" must be an "integer", not "{param.type}"')
                    raise DictConsistencyError(msg, 18, check.xmlfiles)
                for target in check.target:
                    if param.name == 'mini':
                        target.name.min_number = int(param.text)
                    elif param.name == 'maxi':
                        target.name.max_number = int(param.text)
                    else:
                        msg = _(f'unknown parameter "{param.name}" in check "valid_entier"')
                        raise DictConsistencyError(msg, 19, check.xmlfiles)
        remove_indexes.sort(reverse=True)
        for idx in remove_indexes:
            del self.objectspace.space.constraints.check[idx]

    def convert_check(self) -> None:
        """valid and manage <check>
        """
        for check in self.objectspace.space.constraints.check:
            for target in check.target:
                if not hasattr(target.name, 'validators'):
                    target.name.validators = []
                target.name.validators.append(check)
